@@ -1,6 +1,6 @@
 
 use numpy::{IntoPyArray,
-            PyReadonlyArray2, PyReadonlyArrayDyn,
+            PyReadonlyArray1, PyReadonlyArray2, PyReadonlyArrayDyn,
             PyArray3, PyArray2, PyArray1};
 use pyo3::{pymodule, types::PyModule, PyResult, Python};
 
@@ -76,11 +76,13 @@ fn del_msh(_py: Python, m: &PyModule) -> PyResult<()> {
         py: Python<'a>,
         tri2vtx: PyReadonlyArray2<'a, usize>,
         vtx2xyz: PyReadonlyArray2<'a, f32>) -> &'a PyArray3<f32> {
+        let num_val = vtx2xyz.shape()[1];
         let tri2xyz = del_msh::unindex::unidex_vertex_attribute_for_triangle_mesh(
             &tri2vtx.as_slice().unwrap(),
-            &vtx2xyz.as_slice().unwrap());
+            &vtx2xyz.as_slice().unwrap(),
+            num_val);
         numpy::ndarray::Array3::from_shape_vec(
-            (tri2xyz.len()/9,3,3), tri2xyz).unwrap().into_pyarray(py)
+            (tri2xyz.len()/(3*num_val),3,num_val), tri2xyz).unwrap().into_pyarray(py)
     }
 
     #[pyfn(m)]
@@ -98,5 +100,48 @@ fn del_msh(_py: Python, m: &PyModule) -> PyResult<()> {
             num_elem, elem2dist).unwrap().into_pyarray(py)
     }
 
+    #[pyfn(m)]
+    pub fn areas_of_triangles_of_mesh<'a>(
+        py: Python<'a>,
+        tri2vtx: PyReadonlyArray2<'a, usize>,
+        vtx2xyz: PyReadonlyArray2<'a, f32>
+    ) -> &'a PyArray1<f32> {
+        assert_eq!(vtx2xyz.shape()[1], 3);
+        let tri2area = del_msh::sampling::areas_of_triangles_of_mesh(
+            tri2vtx.as_slice().unwrap(),
+            vtx2xyz.as_slice().unwrap() );
+        numpy::ndarray::Array1::from_shape_vec(
+            tri2vtx.shape()[0], tri2area).unwrap().into_pyarray(py)
+    }
+
+    #[pyfn(m)]
+    pub fn sample_uniform(
+        _py: Python,
+        cumsum_tri2area: PyReadonlyArray1<f32>,
+        val01_a: f32,
+        val01_b: f32) -> (usize, f32, f32) {
+        del_msh::sampling::sample_uniform(
+            cumsum_tri2area.as_slice().unwrap(),
+            val01_a, val01_b)
+    }
+
+    #[pyfn(m)]
+    pub fn extract<'a>(
+        py: Python<'a>,
+        tri2vtx: PyReadonlyArray2<'a, usize>,
+        num_vtx: usize,
+        tri2tri_new: PyReadonlyArray1<'a, usize>,
+        num_tri_new: usize) -> (&'a PyArray1<usize>, usize, &'a PyArray1<usize>)
+    {
+        let (tri2vtx_new, num_vtx_new, vtx2vtx_new) = del_msh::extract::extract(
+            tri2vtx.as_slice().unwrap(), num_vtx,
+            tri2tri_new.as_slice().unwrap(), num_tri_new);
+        (
+            numpy::ndarray::Array1::from_vec(tri2vtx_new).into_pyarray(py),
+            num_vtx_new,
+            numpy::ndarray::Array1::from_vec(vtx2vtx_new).into_pyarray(py)
+        )
+    }
+
     Ok(())
 }
\ No newline at end of file
